package tech.nevets.modbot;

import tech.nevets.modbot.api.commands.*;

import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.List;

public class CommandManager {
    public static final List<ICommand> ALL_COMMANDS = new ArrayList<>();
    public static final List<IPrefixCommand> ALL_PREFIX_COMMANDS = new ArrayList<>();
    public static final List<ISlashCommand> ALL_SLASH_COMMANDS = new ArrayList<>();

    public static void addFromRegistry(List<? extends ICommand> cmds) {
        for (ICommand cmd : cmds) {
            ALL_COMMANDS.add(cmd);
            if (cmd instanceof IPrefixCommand) {
                ALL_PREFIX_COMMANDS.add((IPrefixCommand) cmd);
            }
            if (cmd instanceof ISlashCommand) {
                ALL_SLASH_COMMANDS.add((ISlashCommand) cmd);
            }
        }
    }



    public static void addFromRegistry(CommandRegistry registry) {
        ALL_COMMANDS.addAll(registry.getCommands());
        ALL_PREFIX_COMMANDS.addAll(registry.getPrefixCommands());
        ALL_SLASH_COMMANDS.addAll(registry.getSlashCommands());
    }

    @Nullable
    public static ICommand getCommand(String search) {
        String searchLower = search.toLowerCase();

        for (ICommand cmd : ALL_COMMANDS) {
            if (cmd.getName().equals(searchLower) || cmd.getAliases().contains(searchLower)) {
                return cmd;
            }
        }
        return null;
    }

    public static List<ICommand> getAllCommands() {
        return ALL_COMMANDS;
    }


    @Nullable
    public static IPrefixCommand getPrefixCommand(String search) {
        String searchLower = search.toLowerCase();

        for (IPrefixCommand cmd : ALL_PREFIX_COMMANDS) {
            if (cmd.getName().equals(searchLower) || cmd.getAliases().contains(searchLower)) {
                return cmd;
            }
        }
        return null;
    }

    public static List<IPrefixCommand> getAllPrefixCommands() {
        return ALL_PREFIX_COMMANDS;
    }

    @Nullable
    public static ISlashCommand getSlashCommand(String search) {
        String searchLower = search.toLowerCase();

        for (ISlashCommand cmd : ALL_SLASH_COMMANDS) {
            if (cmd.getName().equals(searchLower) || cmd.getAliases().contains(searchLower)) {
                return cmd;
            }
        }
        return null;
    }

    public static List<ISlashCommand> getAllSlashCommands() {
        return ALL_SLASH_COMMANDS;
    }
}
